# This scripts downloads, prepares and saves the `pbmc3k` data. To preprocess
# the data it uses the Seurat package and instructions from the 'Guided clustering'
# tutorial (https://satijalab.org/seurat/articles/pbmc3k_tutorial.html).

args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

.libPaths()

suppressMessages({
  library(TENxPBMCData)
  library(scater)
  library(Seurat)
  library(scran)
})

set.seed(2572021)

pbmc3k <- readRDS(args$raw_data)

rownames(pbmc3k) <- uniquifyFeatureNames(
  rowData(pbmc3k)$ENSEMBL_ID,
  rowData(pbmc3k)$Symbol_TENx
)

old_rowdata <- rowData(pbmc3k)

counts <- as(counts(pbmc3k), "dgCMatrix")
colnames(counts) <- paste0("Cell", 1:ncol(counts))

pbmc3k <- CreateSeuratObject(
  counts = counts,
  project = "pbmc3k",
  min.cells = 3,
  min.features = 200
)

pbmc3k[["percent.mt"]] <- PercentageFeatureSet(pbmc3k, pattern = "^MT-")

pbmc3k <- subset(
  pbmc3k,
  subset = nFeature_RNA > 200 & nFeature_RNA < 2500 & percent.mt < 5
)

pbmc3k <- NormalizeData(
  pbmc3k,
  normalization.method = "LogNormalize",
  scale.factor = 10000
)

pbmc3k <- FindVariableFeatures(
  pbmc3k,
  selection.method = "vst",
  nfeatures = 2000
)

all_genes <- rownames(pbmc3k)
pbmc3k <- ScaleData(pbmc3k, features = all_genes)

pbmc3k <- RunPCA(pbmc3k, features = VariableFeatures(object = pbmc3k))

pbmc3k <- FindNeighbors(pbmc3k, dims = 1:10)
pbmc3k <- FindClusters(pbmc3k, resolution = 0.5)

pbmc3k <- RunUMAP(pbmc3k, dims = 1:10)
pbmc3k <- RunTSNE(pbmc3k, dims = 1:10)

new_cluster_ids <- c(
  "Naive CD4 T",
  "CD14+ Mono",
  "Memory CD4 T",
  "B",
  "CD8 T",
  "FCGR3A+ Mono",
  "NK",
  "DC",
  "Platelet"
)
names(new_cluster_ids) <- levels(pbmc3k)
pbmc3k <- RenameIdents(pbmc3k, new_cluster_ids)

# Convert back to a SingleCellExperiment
pbmc3k <- as.SingleCellExperiment(pbmc3k)
colLabels(pbmc3k) <- colData(pbmc3k)$ident

# Add back some rowData to prevent downstream (Python) errors.
rowData(pbmc3k) <- rownames(pbmc3k)

# Filter to the top 5000 genes.
dec_pbmc3k <- modelGeneVarByPoisson(pbmc3k)
top_pbmc3k <- getTopHVGs(dec_pbmc3k, n = 2000)
pbmc3k <- pbmc3k[top_pbmc3k, ]

# Compute scran size factors. These are not used for creating the log counts
# but are used in the analysis of CDR vs. size factor and possibly elsewhere.
pbmc3k <- computeSumFactors(pbmc3k, cluster = quickCluster(pbmc3k))

# Create subsets of the data.
pbmc3k_b <- pbmc3k[, pbmc3k$ident == "B"]
pbmc3k_cd8t <- pbmc3k[, pbmc3k$ident == "CD8 T"]
pbmc3k_platelet <- pbmc3k[, pbmc3k$ident == "Platelet"]

# Save all files.
saveRDS(pbmc3k, args$data)

#saveRDS(pbmc3k_b , here::here("data",  "real_data", "pbmc3k-b.rds"))
#saveRDS(pbmc3k_cd8t, here::here("data", "real_data", "pbmc3k-cd8t.rds"))
#saveRDS(pbmc3k_platelet, here::here("data", "real_data", "pbmc3k-platelet.rds"))
